
from dotenv import load_dotenv
import logging
import os
from pydantic import BaseModel, ValidationError
import requests
from requests.exceptions import HTTPError, RequestException

# Carrega o token da variavél de ambiente "API_TOKEN_NIBO"
load_dotenv()
token = os.getenv("API_TOKEN_NIBO")

class ApiParams(BaseModel):
    url_end : str
    params : dict | None


class ApiNibo:

    def __init__(self):
        self.token = token
        self.base_url = "https://api.nibo.com.br/empresas/v1/"


    def api_request(self, url_end: str, params: dict = None) -> dict | None:
        """
            Faz uma requisição GET para a API da Nibo.

            Args:
                url_end: Endpoint específico da API (ex: 'clientes', 'notas').
                params: Parâmetros de query string para a requisição.

            Returns:
                requests.Response.json | None: Dicionário JSON da  resposta da
                    requisição ou None em caso de erro.
            """
        # Valida parâmetros
        parametros_validos = ApiParams(url_end=url_end, params=params)
        url_end = parametros_validos.url_end
        params = parametros_validos.params


        api_url = self.base_url + url_end
        headers = {
            "accept": "application/json",
            "apitoken": self.token,
        }
        try:
            response = requests.get(api_url, headers=headers, params=params)
            response.raise_for_status()
            return response.json()

        # Erros
        except ValidationError as validation_err:
            logging.error(f"Erro de validação: {validation_err}")
        except HTTPError as http_err:
            status = getattr(http_err.response, 'status_code', 'sem status')
            logging.error(
                f"HTTP Nibo ({url_end}): {http_err} - Status code: {status}"
            )
        except RequestException as req_err:
            logging.error(f"requisição Nibo: {req_err}")
        except Exception as err:
            logging.error(f"inesperado na requisição Nibo: {err}")



from datetime import date

from api import ApiNibo


def lista_boletos(data: date = date.today()) -> list[dict]:
    """
    Retorna uma lista com os boletos gerados na data informada usando API do Nibo.

    Esta função consulta a API para verificar se existem boletos gerados na data fornecida.
    A data é comparada com o campo `createAt` dos boletos na API, filtrando por ano, mês e dia.

    Args:
        data (date): Data para verificar se existem boletos gerados. Exemplo: date(2024, 12, 25).

    Retruns:
        list[dict]: lista de boletos gerados, onde cada item é um dicionário com informações do boleto.
            Retorna uma lista vazia caso nenhum boleto seja encontrado na data especificada.
    """

    # Formata o filtro para consulta de acordo com a data
    filtro_data = f"year(createAt) eq {data.year} AND month(createAt) eq {data.month} AND day(createAt) eq {data.day}"

    # Configuração da requisição
    boleto_api_url = f"schedules/credit/promise"
    params = {
        "$filter": filtro_data,
    }

    # Requisição
    # response = api_nibo_request(boleto_api_url, params)
    resposta = ApiNibo().api_request(boleto_api_url, params)

    # Resposta itens no formato JSON (dicionário em python)
    boletos_items = resposta["items"]

    return boletos_items



from api import ApiNibo


class ClientesNibo(ApiNibo):

    def __init__(self, id_nibo: str | dict = None):
        super().__init__()
        self.endpoint = "customers"
        self.cliente = None
        if self.cliente:
            self.seleciona_cliente(id_nibo)

    def seleciona_cliente(self, id_nibo):
        filtro_id = f"id eq {id_nibo}"
        params = {"$filter": filtro_id}
        self.cliente = self.api_request(self.endpoint, params)['items'][0]

    @property
    def nome(self):
        return self.cliente["name"] if self.cliente else None

    @property
    def tel(self):
        return self._tel()

    @property
    def email(self):
        return self._email()

    @property
    def cnpj_cpf(self):
        return self._cnpj_cpf()

    def _tel(self):
        try:
            contatos = self.cliente["communication"]
            numero_contato = None
            if contatos:
                telefone = contatos.get("phone")
                celular = contatos.get("cellPhone")
                if telefone:
                    numero_contato = telefone
                elif celular:
                    numero_contato = celular
            return numero_contato
        except Exception as e:
            print(f"Erro no contato - {e}")
            return

    def _email(self):
        try:
            contatos = self.cliente["communication"]
            email = self.cliente.get("email")
            if contatos:
                email = contatos.get("email")
            return email
        except Exception as e:
            print(f"Erro email - {e}")
            return

    def _cnpj_cpf(self):
        try:
            documento = self.cliente["document"]
            if documento:
                cnpj_cpf = documento["number"]
                return cnpj_cpf
        except Exception as e:
            print(f"Erro no documento (cnpj/cpf) - {e}")
            return


def lista_todos_clientes() -> list[dict]:
    # Requisição à API do Nibo
    endpoint = "customers"
    resposta = ApiNibo().api_request(endpoint)

    # Atribui a quantidade total de clientes existentes
    qtd_clientes = resposta["count"]

    # Lista para armazenar todos os clientes encontrados
    todos_os_clientes = []

    # Inicializa a variável página para requisição
    pagina = 0

    # Faz requisição até a quantidade de clientes na lista seja igual à
    # quantidade total de clientes definida na requisição inicial
    while len(todos_os_clientes) != qtd_clientes:

        # Define os parâmetros da requisição (paginamento)
        # "$orderBy" organiza os clientes por nome e "$skip" pula os clientes
        # já retornados na página anterior
        params = {"$orderBy": "name", "$skip": pagina}

        # Faz a requisição para a API e obtém a resposta
        resposta = ApiNibo().api_request(endpoint, params)

        # Armazena os dados da resposta (clientes da página)
        dados = resposta["items"]

        # Adiciona os clientes da página à lista todos_os_clientes
        for item in dados:
            todos_os_clientes.append(item)

        # Atualiza a variável da página para a próxima página de resultados
        # A cada requisição, avança 500 clientes para o próximo lote
        pagina += 500  # 500 é o máximo permitido por página pela API

    return todos_os_clientes


print(lista_todos_clientes())